#ifndef PDV_VECLIB
#define PDV_VECLIB

#include <immintrin.h>

class float256 {
private:
	__m256 data;

public:
	inline static float256 load_from(float * addr) {
		return float256(_mm256_loadu_ps(addr));
	}

	inline static float256 constant(float value) {
		return float256(_mm256_set1_ps(value));
	}

	inline float256(__m256 vec) : data(vec) {}
	inline float256() : data(_mm256_setzero_ps()) {}

	inline float256 operator+(float256 other) {
		return float256(_mm256_add_ps(data, other.data));
	}

	inline float256 operator-(float256 other) {
		return float256(_mm256_sub_ps(data, other.data));
	}

	inline float256 operator*(float256 other) {
		return float256(_mm256_mul_ps(data, other.data));
	}

	inline float256 operator/(float256 other) {
		return float256(_mm256_div_ps(data, other.data));
	}

	inline void store_to(float * addr) {
		_mm256_storeu_ps(addr, data);
	}
};

inline float256 exp(float256 data) {
  static float256 three = float256::constant(3.0f);
  auto addthree = data + three;
  auto subthree = data - three;

  return (addthree*addthree + three) / (subthree*subthree + three);
}

#endif